"""quixote.config

Quixote configuration information.  This module provides both the
default configuration values, and some code that Quixote uses for
dealing with configuration info.  You should not edit the configuration
values in this file, since your edits will be lost if you upgrade to a
newer Quixote version in the future.  However, this is the canonical
source of information about Quixote configuration variables, and editing
the defaults here is harmless if you're just playing around and don't
care what happens in the future.
"""

__revision__ = "$Id: config.py,v 1.16 2000/12/08 22:46:10 gward Exp $"


# Note that the default values here generally prefer convenience and
# verbosity over security.  The assumption is that Quixote is more
# likely to be installed by web developers who want to play around with
# it than web administrators who just want it to go to work on day 1.
# Third-party Quixote applications should ship with more secure
# defaults, so that end-users can install Quixote and that application
# and not have to worry about security implications of the Quixote
# configuration.  If you want to be sure to run a tight ship, consider
# changing DISPLAY_EXCEPTIONS and SECURE_ERRORS.


# The root namespace is the Python namespace that will handle all
# requests to this Quixote application.  This is usually the name of a
# package or module, but could be a class instance or module object --
# any Python namespace!  (Allowing strings is a special case.)
ROOT_NAMESPACE = 'quixote.demo'

# Prefix of URLs that will be directed to Quixote.  This just tells
# Quixote where this application lives in your web site's URL-space;
# you'll also have to configure your web server so that requests to this
# URL go to the FCGI/CGI script for this Quixote application.
URL_PREFIX = '/q'
<<<<<<< config.py

# E-mail address to send errors to
# (None to send no mail at all)

#ERROR_EMAIL = None
ERROR_EMAIL = 'akuchlin@mems-exchange.org'
=======
>>>>>>> 1.16

<<<<<<< config.py
# Filename for logging debugging output (None for no log file.)
=======
# E-mail address to send errors to; None to send no mail at all.  This
# should probably be the email address of your web administrator.
ERROR_EMAIL = None
#ERROR_EMAIL = 'akuchlin@mems-exchange.org'

# Filename for logging debugging output; None for no debug log file.
DEBUG_LOG = None
#DEBUG_LOG = "/www/log/quixote-debug.log"
>>>>>>> 1.16

<<<<<<< config.py
#DEBUG_LOG = None
DEBUG_LOG = "/tmp/quixote-debug.log"

# Page shown when there's an access-control violation
=======
# Page shown when there's an access-control violation (relative to
# URL_PREFIX).
>>>>>>> 1.16
ACCESS_VIOLATION_PAGE = '/access_error'

# Filename for logging error messages (None for output to standard error,
# which should wind up in the Web servers log file.)

<<<<<<< config.py
#ERROR_LOG = None
ERROR_LOG = "/www/httpd/logs/quixote.log"
=======
# Filename for logging error messages; if None, error messages will be
# sent to standard error, so should wind up in the Web server's error
# log file.
ERROR_LOG = None
#ERROR_LOG = "/www/log/quixote-error.log"
>>>>>>> 1.16

# If true, exceptions will also be returned to the browser; otherwise,
# a generic error display will be used.
DISPLAY_EXCEPTIONS = 1

# If true, Quixote will service exactly one request at a time, and
# then exit.  This makes no difference when you're running as a
# straight CGI script, but it makes it easier to debug while running
# as a FastCGI script.
RUN_ONCE = 1

# If true, then any "resource not found" errors will result in a
# consistent, terse, mostly-useless message.  If false, then the
# exact cause of failure will be returned.
SECURE_ERRORS = 0

# Session-related variables
# =========================

# Domain to specify in setting cookies to hold the session ID
COOKIE_DOMAIN = ".example.com"

# Name and path of the cookie that will hold the ID
COOKIE_NAME = "Qsession"
COOKIE_PATH = "/"
