#!/usr/bin/env python

# Setup script for Quixote

from distutils.core import setup

setup (name = "Quixote",
       version = "0.2",
       description = "Quixote Web application development system",
       author = "MEMS Exchange",
       author_email = "quixote@mems-exchange.org",
       url = "http://www.mems-exchange.org/software/python/quixote/",

       package_dir = {'quixote':'.'},
       packages = ['quixote',  'quixote.demo', 'quixote.test',
                   'quixote.util', 'quixote.zope'],
      )

 
